/*****************************************************************
 * Sharp PC-G850V(S) ESP32 Interface
 * By: ChrisHerman
 * Date: December 5th, 2021
 *
 * Receives files from Sharp G850 and sends to PC via USB-Serial (Nano)
 * Requires G850's RTS and CTS lines to be connected witch each other 
 * and pulled-up to +5V. otherwise SHarp will wait forever.
 * 
 *
 * Hardware Hookup:
 * * 
 * Sharp G850 11 pin port         ESP32
 *=========================================
 *  GND (pin 3) __________________ GND
 * 
 * Rx (pin 6)  __________________ GPIO4
 * 
 * Tx (pin 7)  __________________ GDIO5
 * 
 * RTS (pin 4) __________
 *                       |
 * CTS (pin 9) __________+
 *                       |
 * +5V         ---[10K]--+
 * 
 *
 * This code is released under the [MIT License](http://opensource.org/licenses/MIT).
 *****************************************************************/



#include <SoftwareSerial.h>
#include <ESP8266WiFi.h>
#include <ESP8266mDNS.h>
#include <ArduinoOTA.h>




//#define RX_PIN 13 
//#define TX_PIN 12 
#define RX_PIN 5
#define TX_PIN 4 


#define ST(A) #A
#define STR(A) ST(A)



SoftwareSerial SoftSerial(RX_PIN, TX_PIN, true); // RX, TX, inverse_logic = true

// reading buffor config
#define BUFFER_SIZE 1024
  
// global objects
WiFiServer server(23);
byte buff[BUFFER_SIZE];


void checkFlash(){
  uint32_t realSize = ESP.getFlashChipRealSize();
  uint32_t ideSize = ESP.getFlashChipSize();
  FlashMode_t ideMode = ESP.getFlashChipMode();

  Serial.printf("\n");
  Serial.printf("Flash real id:   %08X\n", ESP.getFlashChipId());
  Serial.printf("Flash real size: %u bytes\n", realSize);

  Serial.printf("Flash ide  size: %u bytes\n", ideSize);
  Serial.printf("Flash ide speed: %u Hz\n", ESP.getFlashChipSpeed());
  Serial.printf("Flash ide mode:  %s\n", (ideMode == FM_QIO ? "QIO" : ideMode == FM_QOUT ? "QOUT" : ideMode == FM_DIO ? "DIO" : ideMode == FM_DOUT ? "DOUT" : "UNKNOWN"));

  if (ideSize != realSize) {
    Serial.println("Flash Chip configuration wrong!\n");
  } else {
    Serial.println("Flash Chip configuration ok.\n");
  }

}


void OTASetup(){

   ArduinoOTA.setPassword((const char *)STR(OTAPW));

    ArduinoOTA.onStart([]() {
    Serial.println("Start");
  });
  ArduinoOTA.onEnd([]() {
    Serial.println("\nEnd");
  });
  ArduinoOTA.onProgress([](unsigned int progress, unsigned int total) {
    Serial.printf("Progress: %u%%\r", (progress / (total / 100)));
  });
  ArduinoOTA.onError([](ota_error_t error) {
    Serial.printf("Error[%u]: ", error);
    if (error == OTA_AUTH_ERROR) Serial.println("Auth Failed");
    else if (error == OTA_BEGIN_ERROR) Serial.println("Begin Failed");
    else if (error == OTA_CONNECT_ERROR) Serial.println("Connect Failed");
    else if (error == OTA_RECEIVE_ERROR) Serial.println("Receive Failed");
    else if (error == OTA_END_ERROR) Serial.println("End Failed");
  });
  ArduinoOTA.begin();

 
}











void setup() {
  SoftSerial.begin(SOFTBAUDRATE);
  Serial.begin(BAUDRATE);
  delay(5000);

  checkFlash();

  Serial.println("Connecting WiFI:");
  WiFi.begin(STR(SSID), STR(WIFIPASSWORD));
  while (WiFi.status() != WL_CONNECTED) {
    Serial.print(".");
    delay(500);
  }
  Serial.println("Connected");
  WiFi.setHostname(STR(WIFIHOSTNAME));
  WiFi.setAutoReconnect(true);
  WiFi.persistent(true);
  delay(1000);

  Serial.print("Hostname:");
  Serial.println(WiFi.getHostname());
  Serial.println("Ready");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());


  OTASetup();

  server.begin();
  Serial.println("server started");
  
}

void loop() {
 


  ArduinoOTA.handle();
  

  // wait for client
  WiFiClient client = server.available();
  if (!client)
  return;
  
  //clear any bytes so far in the Serial Buffer
  while(SoftSerial.available()>0)  {  
    SoftSerial.read();  
  }


  while (client.connected()) {
    int size = 0;
  
    // read data from wifi client and send to serial
    while ((size = client.available())) {
           size = (size >= BUFFER_SIZE ? BUFFER_SIZE : size);
           client.read(buff, size);
           SoftSerial.write(buff, size);
           SoftSerial.flush();
    }
  
    // read data from serial and send to wifi client
    while ((size = SoftSerial.available())) {
           size = (size >= BUFFER_SIZE ? BUFFER_SIZE : size);
           SoftSerial.readBytes(buff, size);
           client.write(buff, size);
           client.flush();
    }
  }
  client.stop();


}
